# album.tcl --
# Incr Tcl version of htmlalbum backend.

# $Id: album.tcl,v 1.2 2004/09/21 10:34:31 davidw Exp $

package provide album 0.1

set auto_path "[file dir [info script]] $auto_path"

package require Itcl
package require xml
package require xml::c
package require thumbnails
package require image

set scriptdir [file normalize [file dirname [info script]]]

::itcl::class Album {

    public variable title "No Album"
    # A description of the images in question.
    public variable description ""
    public variable indextemplate [file join $::scriptdir index.tml]
    public variable pagetemplate [file join $::scriptdir page.tml]

    # The htmlalbum.xml file related to this object.
    public variable xmlfile ""

    private variable Images {}
    private variable directory ""

    constructor {args} {
	set xmlfile ""
	eval configure $args
    }

    destructor {}

    public method setdirectory { } {
	if { $xmlfile != "" } {
	    cd [file dirname $xmlfile]
	}
    }

    public method empty {} {
	if { [llength $Images] < 1 } {
	    return 1
	} else {
	    return 0
	}
    }

    public method getfiles { newfiles } {
	foreach f $newfiles {
	    newimage $f
	}
    }

    public method newimage { imagefile } {
	set i [Image \#auto]
	$i fromfile $imagefile
	insert $i
    }

    public method insert {img {position end}} {
	set Images [linsert $Images $position $img]
    }

    public method remove {position} {
	set i [lindex $Images $position]
	set Images [lreplace $Images $position $position]
	::itcl::delete object $i
    }

    public method move {from to} {
	set fromi [lindex $Images $from]
	if { $from <= $to } {
	    incr to
	} else {
	    incr from
	}
	set Images [lreplace [linsert $Images $to $fromi] $from $from]
    }

    public method order {args} {
	set neworder {}
	foreach p $args {
	    lappend neworder [lindex $Images $p]
	}
	set Images $neworder
    }

    public method imagecget {index var} {
	set i [lindex $Images $index]
	return [$i cget $var]
    }

    public method imageconfigure {index var val} {
	set i [lindex $Images $index]
	return [$i configure $var $val]
    }

    public method allcget {var} {
	set retval {}
	foreach i $Images {
	    lappend retval [$i cget $var]
	}
	return $retval
    }

    public method print {} {
    }

    public method writexml {} {
	set dir [file dirname $xmlfile]
	if { ! [file exists $dir] } {
	    file mkdir $dir
	}
	set f [open $xmlfile "w"]
	puts $f "<?xml version=\"1.0\"?>
     <gallery title=\"$title\">"
	puts $f "<description>"
	puts $f $description
	puts $f "</description>"
	foreach i $Images {
	    
	    puts $f "
       <page filename=\"[$i cget -htmlfile]\" title=\"[$i cget -title]\">
	 <photo src=\"[$i cget -filename]\"></photo>"

	    foreach key [$i cget -keywords] {
		puts $f "         <key>[string trim $key]</key>"
	    }

	    puts $f "         <caption>[$i cget -caption]</caption>
       </page>"

	}
	puts $f "</gallery>"
	close $f
    }

    # writehtml --
    #
    #	Create index with thumbnails.

    public method writehtml { index callback } {
	set fl [open $index "w"]
	set tfl [open $indextemplate]
	set indextemplatedata [read $tfl]
	close $tfl

	set len [llength $Images]
	incr len 2

	set j 0
	foreach i $Images {
	    $callback $j $len
	    incr j

	    set fn [$i cget -filename]
	    # This fills in thumbnail names if they don't exist.
#	    set thumbdir [file join [file rootname $fn] thumbs]
	    set thumbdir thumbs
	    if { ! [file isdirectory $thumbdir] } {
		file mkdir $thumbdir
	    }

	    set tn [$i cget -thumbnail]
	    if { $tn == "" } {
		set tn [file join $thumbdir $fn]
		$i configure -thumbnail $tn
	    }
	    thumbnails::makethumb $fn $tn
	}
	set int [interp create]
	interp eval $int [list package require html]
	interp eval $int [list set title $title]
	interp eval $int [list set htmlfiles [allcget -htmlfile]]
	interp eval $int [list set titles [allcget -title]]
	interp eval $int [list set thumbnails [allcget -thumbnail]]
	set res [interp eval $int [list subst $indextemplatedata]]
	interp delete $int
	unset int
	puts $fl $res
	close $fl


	catch {
	    file copy -force [file join $::scriptdir style.css] .
	}
	catch {
	    file copy [file join $::scriptdir right.png] .
	    file copy [file join $::scriptdir left.png] .
	    file copy [file join $::scriptdir up.png] .
	}

	set tfl [open $pagetemplate]
	set pagetemplatedata [read $tfl]
	close $tfl

	set idx 0
	set lastimage [expr {[llength $Images] - 1}]
	set int [interp create]

	foreach i $Images {
	    set prev ""
	    set next ""
	    set prevtitle ""
	    set nexttitle ""

	    set fl [open [$i cget -htmlfile] w]
	    # If it's the first image.
	    if { $idx > 0 } {
		set p [lindex $Images [expr {$idx - 1}]]
		set prev [$p cget -htmlfile]
		set prevtitle [$p cget -title]
	    }
	    # If it's the last image.
	    if { $idx < $lastimage } {
		set n [lindex $Images [expr {$idx + 1}]]
		set next [$n cget -htmlfile]
		set nexttitle [$n cget -title]
	    }
	    set caption [string trim [$i cget -caption]]

	    interp eval $int [list package require html]
	    interp eval $int [list set title [$i cget -title]]
	    interp eval $int [list set prev $prev]
	    interp eval $int [list set next $next]
	    interp eval $int [list set prevtitle $prevtitle]
	    interp eval $int [list set nexttitle $nexttitle]
	    interp eval $int [list set index $index]
	    interp eval $int [list set filename [$i cget -filename]]
	    interp eval $int [list set caption [$i cget -caption]]

	    set res [interp eval $int [list subst $pagetemplatedata]]

	    puts $fl $res
	    close $fl

	    incr idx
	}

	$callback $j $len

	interp delete $int
	unset int
    }

    public method getxml { {xmlfilename "htmlalbum.xml"} } {
	set xmlfile $xmlfilename
	set fl [open $xmlfilename]
	set data [read $fl]
	close $fl

	array set element {caption 0 key 0}
	set parser [::xml::parser \
			-elementstartcommand [list $this element start] \
			-elementendcommand [list $this element end] \
			-characterdatacommand [list $this pcdata] \
			-final 1]
	$parser parse $data
    }

    private variable currentimage
    private variable element

    public method element {tag name {attrs {}} args} {
	array set attrhash $attrs
	if { $tag == "start" } {
	    switch -exact $name {
		"description" {
		    set element(description) 1
		}
		"page" {
		    set currentimage [Image \#auto]
		    $currentimage configure -htmlfile $attrhash(filename)
		    $currentimage configure -title $attrhash(title)
		}
		"key" {
		    set element(key) 1
		}
		"caption" {
		    set element(caption) 1
		}
		"photo" {
		    $currentimage configure -filename $attrhash(src)
		}
		"gallery" {
		    set title $attrhash(title)
		}
		"thumbnail" {
		    $currentimage configure -thumbnail $attrhash(filename)
		}
	    }
	} elseif { $tag == "end" } {
	    switch -exact $name {
		"caption" {
		    set element(caption) 0
		}
		"key" {
		    set element(key) 0
		}
		"page" {
		    lappend Images $currentimage
		}
		"gallery" {
		}
	    }
	}
    }

    public method pcdata { pcdata } {
	set stripped [string trim $pcdata]
	if { $element(caption) == 1 && $stripped != "" } {
	    set caption [$currentimage cget -caption]
	    append caption "$stripped "
	    $currentimage configure -caption $caption
	}
	if { $element(key) == 1 } {
	    set keywords [$currentimage cget -keywords]
	    lappend keywords $stripped
	    $currentimage configure -keywords $keywords
	}
    }
}
